/**
 * Generated from 'examples/jsm/postprocessing/GlitchPass.js'
 */

(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('three'), require('examples/jsm/postprocessing/Pass.js'), require('examples/jsm/shaders/DigitalGlitch.js')) :
	typeof define === 'function' && define.amd ? define(['exports', 'three', 'examples/jsm/postprocessing/Pass.js', 'examples/jsm/shaders/DigitalGlitch.js'], factory) :
	(global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.THREE = global.THREE || {}, global.THREE, global.THREE, global.THREE));
}(this, (function (exports, THREE, Pass_js, DigitalGlitch_js) { 'use strict';

	/**
	 * @author alteredq / http://alteredqualia.com/
	 */

	var GlitchPass = function ( dt_size ) {

		Pass_js.Pass.call( this );

		if ( DigitalGlitch_js.DigitalGlitch === undefined ) console.error( "GlitchPass relies on DigitalGlitch" );

		var shader = DigitalGlitch_js.DigitalGlitch;
		this.uniforms = THREE.UniformsUtils.clone( shader.uniforms );

		if ( dt_size == undefined ) dt_size = 64;


		this.uniforms[ "tDisp" ].value = this.generateHeightmap( dt_size );


		this.material = new THREE.ShaderMaterial( {
			uniforms: this.uniforms,
			vertexShader: shader.vertexShader,
			fragmentShader: shader.fragmentShader
		} );

		this.fsQuad = new Pass_js.Pass.FullScreenQuad( this.material );

		this.goWild = false;
		this.curF = 0;
		this.generateTrigger();

	};

	GlitchPass.prototype = Object.assign( Object.create( Pass_js.Pass.prototype ), {

		constructor: GlitchPass,

		render: function ( renderer, writeBuffer, readBuffer /*, deltaTime, maskActive */ ) {

			this.uniforms[ "tDiffuse" ].value = readBuffer.texture;
			this.uniforms[ 'seed' ].value = Math.random();//default seeding
			this.uniforms[ 'byp' ].value = 0;

			if ( this.curF % this.randX == 0 || this.goWild == true ) {

				this.uniforms[ 'amount' ].value = Math.random() / 30;
				this.uniforms[ 'angle' ].value = THREE.Math.randFloat( - Math.PI, Math.PI );
				this.uniforms[ 'seed_x' ].value = THREE.Math.randFloat( - 1, 1 );
				this.uniforms[ 'seed_y' ].value = THREE.Math.randFloat( - 1, 1 );
				this.uniforms[ 'distortion_x' ].value = THREE.Math.randFloat( 0, 1 );
				this.uniforms[ 'distortion_y' ].value = THREE.Math.randFloat( 0, 1 );
				this.curF = 0;
				this.generateTrigger();

			} else if ( this.curF % this.randX < this.randX / 5 ) {

				this.uniforms[ 'amount' ].value = Math.random() / 90;
				this.uniforms[ 'angle' ].value = THREE.Math.randFloat( - Math.PI, Math.PI );
				this.uniforms[ 'distortion_x' ].value = THREE.Math.randFloat( 0, 1 );
				this.uniforms[ 'distortion_y' ].value = THREE.Math.randFloat( 0, 1 );
				this.uniforms[ 'seed_x' ].value = THREE.Math.randFloat( - 0.3, 0.3 );
				this.uniforms[ 'seed_y' ].value = THREE.Math.randFloat( - 0.3, 0.3 );

			} else if ( this.goWild == false ) {

				this.uniforms[ 'byp' ].value = 1;

			}

			this.curF ++;

			if ( this.renderToScreen ) {

				renderer.setRenderTarget( null );
				this.fsQuad.render( renderer );

			} else {

				renderer.setRenderTarget( writeBuffer );
				if ( this.clear ) renderer.clear();
				this.fsQuad.render( renderer );

			}

		},

		generateTrigger: function () {

			this.randX = THREE.Math.randInt( 120, 240 );

		},

		generateHeightmap: function ( dt_size ) {

			var data_arr = new Float32Array( dt_size * dt_size * 3 );
			var length = dt_size * dt_size;

			for ( var i = 0; i < length; i ++ ) {

				var val = THREE.Math.randFloat( 0, 1 );
				data_arr[ i * 3 + 0 ] = val;
				data_arr[ i * 3 + 1 ] = val;
				data_arr[ i * 3 + 2 ] = val;

			}

			return new THREE.DataTexture( data_arr, dt_size, dt_size, THREE.RGBFormat, THREE.FloatType );

		}

	} );

	exports.GlitchPass = GlitchPass;

})));
